unit UnitThreadNotification;

interface

uses
  Classes, Windows, SysUtils;

type
  TThreadNotification = class(TThread)
     procedure Go(NomDuDossier:String; CisSousRepertoire :Boolean;
                  FiltreNotification: DWORD);
    public
      destructor Destroy; override;
    private
      FNomDossier:string;
      FCisSousRepertoire:Boolean;
      FFiltreNotification: DWORD;
      HandleNotification:THandle;
    protected
      procedure OnResultat;
      procedure Execute; override;
  end;

procedure DestructionNotification;
procedure CreationNotification(NomDossier:string; CisSouDossier:Boolean;
                               FiltreNotification : DWORD);
var     ThreadNotification:TThreadNotification;

implementation

uses Unit1, Dialogs;

function RetireSlash(const S: string): string;
// retire le \  la fin s'il existe sauf si on est  la racine (c:\)
begin
  Result := S;
  if (copy(Result, length(Result)-1,2)<>':\')
  and (IsPathDelimiter(Result, Length(Result))) then
    SetLength(Result, Length(Result)-1); 
end;

procedure CreationNotification(NomDossier:string; CisSouDossier:Boolean;
                               FiltreNotification : DWORD);
begin
 if ThreadNotification<>nil then DestructionNotification;
 ThreadNotification:=TThreadNotification.Create(True);
 { True : cration suspendue  le process ne s'excute pas encore }
 ThreadNotification.Go(NomDossier, CisSouDossier, FiltreNotification);
end;



procedure DestructionNotification;
begin
   if ThreadNotification<>nil then
   begin
     ThreadNotification.Terminate;
     // attente de la fin du Thread Ferme quand mme au bout de 10 secondes
     WaitForSingleObject(ThreadNotification.Handle,10000);
     ThreadNotification:=nil;
   end;

end;

{ ThreadNotification }



destructor TThreadNotification.Destroy;
begin
  if HandleNotification<>0 then FindCloseChangeNotification(HandleNotification);
  inherited;
end;

procedure TThreadNotification.OnResultat;
begin
   Form1.Memo1.Lines.Add('Changement le '+ DateToStr(Now)+'  '+ TimeToStr(Now));
end;

procedure TThreadNotification.Execute;
begin
  //initialisation : on indique  Windows le filtre correspondant  ce que
  //l'on veut suivre
  HandleNotification := FindFirstChangeNotification(
          // Chemin  surveiler mais sans \  la fin
          PChar(RetireSlash(FNomDossier)),
          //  surveiller aussi les sous-rpertoires ou pas
          LongBool(ord(FCisSousRepertoire)),
          FFiltreNotification
          );

  While not Terminated do
  begin
      // C'est ainsi que l'on voit s'il y a eu un changement
    if WaitForSingleObject(HandleNotification,200) =  WAIT_OBJECT_0
    then
    begin
      FindNextChangeNotification(HandleNotification);// pour avoir les suivants
      Synchronize(OnResultat);
      // l'idal serait de connaitre le nom du fichier qui a chang.
      // pour a, il faudrait explorer le dossier (et sous-dossiers au besoin)
      // (taper FindFirst dans le moteur de recherche de http://www.phidels.com)
      // regardez en particulier le composant ScruteDossiers.
      // voir aussi l'API ReadDirectoryChangesW
    end;
  end;
  // Indique  Windows que l'on arrte de demander une notification
  // lorsqu'il y a un changement
  FindCloseChangeNotification(HandleNotification);
  HandleNotification:=0;
end;



procedure TThreadNotification.Go(NomDuDossier:String; CisSousRepertoire :Boolean;
                              FiltreNotification: DWORD);
begin
  FNomDossier:=NomDuDossier;
  FCisSousRepertoire:=CisSousRepertoire;
  FFiltreNotification:=FiltreNotification;
  FreeOnTerminate:=True; //on veut une destruction automatique aprs excution.
  {valeurs possibles pour Priority :
  tpIdle	Le thread ne s'excute que lorsque le systme est inoccup.
  Windows n'interrompt pas les autres threads pour excuter un thread de priorit tpIdle.
  tpLowest	La priorit du thread est deux points en dessous de la normale.
  tpLower	La priorit du thread est un point en dessous de la normale.
  tpNormal	La priorit du thread est normale.
  tpHigher	La priorit du thread est un point au dessus de la normale
  tpHighest	La priorit du thread est deux points au dessus de la normale.
  tpTimeCritical	Le thread a la priorit la plus leve.}
  Priority:= tpNormal;
  Resume;    { maintenant, le thread est excut }
end;

initialization

finalization
// cette partie s'excute  l'arrt du programme.
   DestructionNotification;
end.



{Extrait de l'aide :
FILE_NOTIFY_CHANGE_FILE_NAME	
Any filename change in the watched directory or subtree causes a change
notification wait operation to return. Changes include renaming, creating,
or deleting a filename.
FILE_NOTIFY_CHANGE_DIR_NAME	
Any directory-name change in the watched directory or subtree causes a change
notification wait operation to return. Changes include creating or deleting a directory.
FILE_NOTIFY_CHANGE_ATTRIBUTES	
Any attribute change in the watched directory or subtree causes a change
notification wait operation to return.
FILE_NOTIFY_CHANGE_SIZE	
Any file-size change in the watched directory or subtree causes a change
notification wait operation to return. The operating system detects a change in
file size only when the file is written to the disk. For operating systems that
use extensive caching, detection occurs only when the cache is sufficiently flushed.
FILE_NOTIFY_CHANGE_LAST_WRITE	
Any change to the last write-time of files in the watched directory or subtree
causes a change notification wait operation to return. The operating system detects
a change to the last write-time only when the file is written to the disk.
For operating systems that use extensive caching, detection occurs only when
the cache is sufficiently flushed.
FILE_NOTIFY_CHANGE_SECURITY	
Any security-descriptor change in the watched directory or subtree causes a change
notification wait operation to return.}



